﻿Imports System.Collections.Generic
Imports System.Linq
Imports System.Data.Common
Imports System.ComponentModel
Imports System.Reflection
Imports System.Windows.Forms
Imports System.Text
Imports System.Text.RegularExpressions

#Region "NoCompile"
#If NoCompile Then
TableAdapter wrappert einen DbDataAdapter, mit dem Dataset-Tables befüllt und rückgespeichert wern können.
Fürs Befüllen wird großer Aufwand getrieben, damit "halbes" Sql angegeben werden kann, welches den Select spezifiziert. Also der Select-Abschnitt ist ja fix - ergibt sich aus den Datacolumns der DataTable. Aber im From-, Where- und annere Abschnitten kann man schreiben was man will. Etwa folgendes befüllt die BestellDetails mit allen BestellDetails aller Bestellungen eines Kunden:
SELECT [Order Details].* FROM ([Orders] INNER JOIN [Order Details] 
ON ([Orders].[CustomerID]=@p15 AND [Orders].[OrderID]=[Order Details].[OrderID])) 
Also dassis generiert aus der recht einfachen Anweisung: 
rwCustomer.FillChildTables(.Order_Details)
also eine Customerrow befüllt ihre Child-Child-Table "OrderDetails". Braucht man in BindingSource_CurrentChanged-Handlern.
#End If
#End Region 'NoCompile

Namespace System.Data

   <DebuggerDisplay("{Table.TableName}")> _
   Public NotInheritable Class TableAdapter

      Public WithEvents Table As DataTable
      Public ReadOnly Adapter As DbDataAdapter
      Public AllCommands As New List(Of DbCommand)()
      Public ChildTables As DataTable() 'in TopDown-Order
      Private _DtsAttach As DatasetAdapter
      Public ReadOnly Rank As Integer

      Public Function Update() As Integer
         Return Adapter.Update(Table)
      End Function
      Public Function Update(ByVal rows As DataRow()) As Integer
         Return Adapter.Update(rows)
      End Function

      Private Function FillCore(ByVal cmd As DbCommand, ByVal clearBeforeFill As Boolean) As Integer
         EnableEvents = False
         Try
            If clearBeforeFill Then
               For i = ChildTables.Count - 1 To 0 Step -1
                  If ChildTables(i).Rows.Count > 0 Then ChildTables(i).Clear()
               Next
            End If
#If DEBUG Then
            Dbg(cmd.CommandText)
            Dim s = ", ".Between( _
               From p In cmd.Parameters.Cast(Of DbParameter)() _
               Select p.ParameterName.And("-", p.SourceColumn, ": ", p.Value))
            If s.Length > 0 Then Dbg(s)
#End If
            Adapter.SelectCommand = cmd
            FillCore = Adapter.Fill(Table.DataSet)
         Catch ex As ConstraintException
            For i = 0 To Table.Rows.Count - 1
               If Table.Rows(i).HasErrors Then
                  Dim rw = Table.Rows(i)
                  Dim cols = Table.Columns.Cast(Of DataColumn).ToArray
                  Stop
                  Throw New ConstraintException("Fehler in Zeile ".And(i, ": ", rw.RowError), ex)
               End If
            Next
         Finally
            EnableEvents = True
         End Try
      End Function

      Friend Function Fill(ByVal conditions As String, ByVal paramValues As IEnumerable, ByVal clearBeforeFill As Boolean, ByVal distinct As Boolean) As FillJob
         Using cmd = _DtsAttach.CreateSelectCommand(Table, distinct, conditions, paramValues)
            Return New FillJob(Me, cmd, clearBeforeFill)
         End Using
      End Function

      Private Shared _rgxWhere As New Regex("^\s*where\s(.*)", RegexOptions.IgnoreCase)
      Friend Function FillByParentRow(ByVal rwCriteria As DataRow, ByVal conditions As String, ByVal values() As Object) As FillJob
         Dim rls = rwCriteria.Table.GetRelationPathTo(Table).GetEnumerator
         If Not rls.MoveNext Then Throw Me.Exception("keine DataRelation vorhanden")
         Dim rl = rls.Current
         Dim primCol = rl.ParentColumns(0)
         Dim criteria = rwCriteria(primCol) 'Criteria ist der PrimWert der ParentColumn der ersten DataRelation
         Dim colInf = New ColumnInfo(rl.ChildColumns(0), Nothing, _DtsAttach.Quote)
         Dim sql As String
         If rls.MoveNext Then
            'mehrstufiger RelationPath: Für die ChildColumn der ersten DataRelation wird Criteria als Bedingung gesetzt. die weiteren DataRelations werden da dran-gejoint
            Dim joint = New Joiner(_DtsAttach.Quote)
            joint.Add(colInf.ColName & "=?", rls.Current)
            Do While rls.MoveNext
               joint.Add(rls.Current)
            Loop
            sql = joint.Sql.And(" ", conditions) 'zuletzt conditions anhängen
         Else
            'nur eine DataRelation: Für die ChildColumn wird Criteria als Bedingung gesetzt.
            sql = "WHERE ".And(colInf.Fullname, "=? ", _
               _rgxWhere.Replace(conditions, "AND ($1)")) 'und conditions anhängen
         End If
         If _DtsAttach.HasChanges() Then _DtsAttach.Save(Nothing)
         Dim allVals = New Object() {criteria}.Concat(values)
         Return Fill(sql, allVals, True, False)
      End Function

      Public Property EnableEvents() As Boolean
         Get
            Return Table.Attach.Enable
         End Get
         Set(ByVal value As Boolean)
            If value Then
               'top-down DataTable-Events re-aktivieren
               For Each tb In ChildTables
                  tb.Attach.EnableComplete(value)
               Next
            Else
               'bottom-up DataTable-Events deaktivieren
               For I As Integer = ChildTables.Length - 1 To 0 Step -1
                  ChildTables(I).Attach.EnableComplete(value)
               Next
            End If
         End Set
      End Property

      Private Sub Table_Disposed(ByVal sender As Object, ByVal e As EventArgs) Handles Table.Disposed
         Adapter.Dispose()
         For Each cmd In AllCommands
            If cmd.NotNull Then cmd.Dispose()
         Next
      End Sub
   End Class   'TableAdapter

End Namespace
