﻿Imports System.Collections.Generic
Imports System.Linq
Imports System.Data.Common
Imports System.Data.OleDb
Imports System.ComponentModel
Imports System.Reflection
Imports System.Windows.Forms
Imports System.Text
Imports System.Text.RegularExpressions
Imports System.Data.SqlServerCe

Namespace System.Data
#Region "NoCompile"
#If NoCompile Then
hier nur Sub New des TableAdapters: Die Konfiguration ist recht kompliziert, denn er muß für jede Spalte ein DBParameterTemplate speichern, damit andernorts aus sql-Ausdrücken parametrisierte Select-Commands konstruiert wern können.
Entnommen wern die ParamTemplates dem InsertCommand, und für Autowert-ID-Spalten dem Delete-Command, denn für Autowert-IDs generiert der CommandBuilder keine Insert-Parameter.
Die ParamTemplates wern mit eindeutigen ParameterNames ausgerüstet - aber jetzt muß auch der generierte CommandText die neuen ParameterNames addressieren, es sei denn, er addressiert Parameter einfach mit '?'.
Und natürlich alle TableNames und ColumnNames der Command-Texte in '[]' wrappern, evtl. bei MySql in ´´.
Dann wird die Save-Strategie vorbereitet: Access und SqlCe schicken jedem Insert einen Select hinterher, um den von der DB vergebenen Primkey zu erfragen. SqlServer unterstützt UpdateRowSource.Both - man kann also einen Select gleich an den InsertCommandText anhängen, und er aktualisiert dann alle Werte, die u.U. ja vonne DB gesetzt wurden.
Bei Primkey vom Typ Guid wird eingerichtet, dass bei TableNewRow gleich ein Guid generiert wird.
#End If
#End Region 'NoCompile

   Partial Class TableAdapter
      Private Shared _BindFlags As BindingFlags = BindingFlags.Instance Or BindingFlags.NonPublic

      Public Event RowUpdated As EventHandler(Of RowUpdatedEventArgs)

      Public Sub New(ByVal tbl As DataTable, ByVal dtsAttach As DatasetAdapter, ByVal rank As Integer)
         Table = tbl
         _DtsAttach = dtsAttach
         Me.Rank = rank
         ChildTables = (New DataTable() {tbl}) _
            .Concat(tbl.GetAllChildRelations.Select(Function(rl) rl.ChildTable)).ToArray
         Dim att = Table.Attach
         Dim builder = dtsAttach.CommandBuilder
         ' when Commandbuilder is set to another DataAdapter, it disposes the commands it had
         '°delivered. For that use command-Clones in Adapter
         Select Case att.Persistance
            Case Persistance.No : Return     ' TableAdapter stays valid, but without function. This
               '°                                               simplyfies general persistance-code
            Case Persistance.Yes, Persistance.Guess
               Adapter = _DtsAttach.Factory.CreateDataAdapter
               Adapter.SelectCommand = _DtsAttach.CreateSelectCommand(Table, False, "", New Object() {})
               builder.DataAdapter = Adapter
               Try
                  Adapter.DeleteCommand = builder.GetDeleteCommand.Clone
                  att.Persistance = Persistance.Yes
               Catch ex As InvalidOperationException
                  If att.Persistance = Persistance.Guess Then    ' TableAdapter stays valid without
                     '°                                                                    function
                     att.Persistance = Persistance.No
                     Adapter.SelectCommand.Dispose()
                     Adapter.Dispose()
                     Return
                  End If
                  Throw Me.Exception(ex, "Can't acces database with:", _
                     Environment.NewLine, Adapter.SelectCommand.CommandText)
               End Try
            Case Else
               Throw Me.Exception("invalid persistance of ", Table.TableName)
         End Select
         With Adapter
            .UpdateCommand = builder.GetUpdateCommand.Clone
            .InsertCommand = builder.GetInsertCommand.Clone
            'get a Param for each Column
            For Each p As DbParameter In .InsertCommand.Parameters
               dtsAttach.AddKnownParam(tbl, p)
            Next
            .TableMappings.Add(DbDataAdapter.DefaultSourceTableName, tbl.TableName)
            dtsAttach.AddKnownParam(tbl, .DeleteCommand.Parameters(0))
            AllCommands.AddRange(NewArray( _
                .SelectCommand, .DeleteCommand, .InsertCommand, .UpdateCommand))
         End With
         If tbl.PrimaryKey.Length <> 1 Then Return
         Dim primCol = tbl.PrimaryKey(0)
         If primCol.Null Then Return
         If primCol.DataType Is GetType(Guid) Then
            AddHandler primCol.Table.TableNewRow, AddressOf _Table_NewRow
         ElseIf primCol.AutoIncrement Then
            _InitAutoIncrement(primCol)
         End If
      End Sub

      Private Sub _InitAutoIncrement(ByVal primCol As DataColumn)
         If _DtsAttach.LastRowIdSqlFunction.Null Then
            ' with some providers you must re-query the primKey in the Adapter.RowUpdated-event
            With TryCast(Adapter, OleDbDataAdapter)
               If .NotNull Then AddHandler .RowUpdated, AddressOf _Adapter_RowUpdated : Return
            End With
            With TryCast(Adapter, SqlCeDataAdapter)
               If .NotNull Then AddHandler .RowUpdated, AddressOf _Adapter_RowUpdated : Return
            End With
         End If
         ' smart providers support appending a special Select-Command at the Insert-Command, to re-
         '°query the autogenerated primaryKey within the same Insert-Command 
         With New ColumnInfo(primCol, Nothing, _DtsAttach.Quote)
            Adapter.InsertCommand.CommandText &= ";".And( _
               "SELECT ", .ColName, " FROM ", .TableName, _
               " WHERE ", .ColName, " = ", _DtsAttach.LastRowIdSqlFunction, "()")
         End With
         Adapter.InsertCommand.UpdatedRowSource = UpdateRowSource.Both
      End Sub

      Private Sub _Adapter_RowUpdated(ByVal sender As Object, ByVal e As RowUpdatedEventArgs)
         RaiseEvent RowUpdated(sender, e)
      End Sub

      Private Sub _Table_NewRow(ByVal sender As Object, ByVal e As DataTableNewRowEventArgs)
         e.Row(e.Row.Table.PrimaryKey(0)) = Guid.NewGuid
      End Sub
   End Class 'Partial Class TableAdapter

End Namespace
