﻿Imports System

Namespace System

   ''' <summary>
   ''' wrappert einen frei definierbaren Initialisierung- und einen Aufräum-Delegaten, sodaß ein Aufruf von "Using ActionFrame.Enter()" sicher initialisiert und aufräumt.
   ''' Bei Verschachtelung wird nur beim Betreten/Verlassen des äußeren Using-Blocks initialisiert/aufgeräumt 
   ''' </summary>
   ''' <remarks></remarks>
   Public Class ActionFrame : Implements IDisposable

      ' Dispose() wird hier "mißbraucht", um den Level runterzuzählen, und bei 0 die Aufräum-Aktion aufzurufen. IDisposable ermöglicht die Verwendung der sehr sicheren Using-Block-Syntax.

      Protected _enter As Action, _exit As Action
      Protected _level As Integer = 0
      ''' <summary>
      ''' usually better call .Enter() within an using-block, to make shure Enter and 
      ''' Exit are called propperly
      ''' </summary>
      Public Sub ExecuteEnter()
         _enter()
      End Sub
      ''' <summary>
      ''' usually better call .Enter() within an using-block, to make shure Enter and 
      ''' Exit are called propperly
      ''' </summary>
      Public Sub ExecuteExit()
         _exit()
      End Sub
      Public Shared Function ExitWith(ByVal [exit] As Action) As IDisposable
         Return (New ActionFrame(Nothing, [exit])).Enter
      End Function
      Public Sub New()
      End Sub
      Public Sub New(ByVal enter As Action, ByVal [exit] As Action)
         Add(enter, [exit])
      End Sub
      Private Sub EnterCore()
         _level += 1
         If _level = 1 AndAlso _enter IsNot Nothing Then _enter()
      End Sub
      Public Function Enter() As IDisposable
         EnterCore()
         Return Me
      End Function
      Private Sub Dispose() Implements IDisposable.Dispose
         _level -= 1
         Select Case _level
            Case -1
               Throw Me.Exception("cannot set Level < 0!")
            Case 0
               _exit()
         End Select
      End Sub
      ''' <summary>
      ''' Zufügen weiterer ActionFrame läßt baumartige Ablauf-Strukturen entstehen
      ''' </summary>
      Public Sub Add(ByVal child As ActionFrame)
         Add(AddressOf child.EnterCore, AddressOf child.Dispose)
      End Sub
      Public Sub Add(ByVal enter As Action, ByVal [exit] As Action)
         enter.Handles(_enter)
         [exit].Handles(_exit)
      End Sub
   End Class

   Public Class ConditionalActionFrame : Inherits ActionFrame
      Protected ToToggle As Boolean
      Protected _condition As Func(Of Boolean), _innerEnter As Action, _innerExit As Action
      Public Sub New(ByVal condition As Func(Of Boolean), ByVal enter As Action, ByVal [exit] As Action)
         _condition = condition
         _innerEnter = enter
         _innerExit = [exit]
         MyBase.Add(AddressOf TryEnter, AddressOf TryExit)
      End Sub
      Private Sub TryEnter()
         ToToggle = _condition()
         If ToToggle Then _innerEnter()
      End Sub
      Private Sub TryExit()
         If ToToggle Then _innerExit()
      End Sub
   End Class

End Namespace
