﻿Imports System.Runtime.CompilerServices
Imports System.Text
Imports System.ComponentModel
Imports System.Text.RegularExpressions
Imports System.IO
Imports System.Data.Common

Namespace System.Data

   Public Module DataTableX

      <Extension()> _
      Public Sub CopyTo(Of T As DataRow)(ByVal subj As T, ByVal other As T)
         Dim itms = subj.ItemArray
         If other.RowState <> DataRowState.Detached Then
            Dim tb = other.Table
            For Each indx In tb.PrimaryKey.Select(Function(cl) cl.Ordinal)
               itms(indx) = other(indx)
            Next
         End If
         other.ItemArray = itms
      End Sub

      <Extension()> _
      Public Function All(Of T As DataRow)(ByVal subj As TypedTableBase(Of T)) As IEnumerable(Of T)
         Return DirectCast(subj, DataTable).All.Cast(Of T)()
      End Function
      <Extension()> _
      Public Function All(ByVal subj As DataTable) As IEnumerable(Of DataRow)
         Return From rw In subj Where 0 = (rw.RowState And (DataRowState.Deleted Or DataRowState.Detached))
      End Function

      <Extension()> _
      Public Function ChildRels(ByVal subj As DataTable) As IEnumerable(Of DataRelation)
         Return subj.ChildRelations.Cast(Of DataRelation)()
      End Function

      <Extension()> _
      Public Function ParentRels(ByVal subj As DataTable) As IEnumerable(Of DataRelation)
         Return subj.ParentRelations.Cast(Of DataRelation)()
      End Function

      <Extension()> _
      Public Function ChildTables(ByVal subj As DataTable) As IEnumerable(Of DataTable)
         Return From rl In subj.ChildRelations.Cast(Of DataRelation)() _
            Where rl.ChildKeyConstraint.NotNull Select rl.ChildTable
      End Function

      <Extension()> _
      Public Function ParentTables(ByVal subj As DataTable) As IEnumerable(Of DataTable)
         Return subj.ParentRelations.Cast(Of DataRelation).Select(Function(rl) rl.ParentTable)
      End Function

      Private _ClearingTables As New HashSet(Of DataTable)
      <Extension()> _
      Public Sub ClearRecursive(ByVal subj As DataTable)
         If subj.Rows.Count = 0 OrElse Not _ClearingTables.Add(subj) Then Return
         subj.ChildTables.ForEach(AddressOf ClearRecursive)
         _ClearingTables.Remove(subj)
         subj.Clear()
      End Sub

      ''' <summary>sort tables, so that no childtable is before its parentTable</summary>
      <Extension()> _
      Public Function GetRankedTables(ByVal dts As DataSet) As List(Of DataTable)
         Return New TopologicSort(dts.Tables.Cast(Of DataTable)).rankedTables
      End Function
      <Extension()> _
      Public Function GetRankedTables(ByVal subj As IEnumerable(Of DataTable)) As List(Of DataTable)
         Return New TopologicSort(subj).rankedTables
      End Function
      Private Class TopologicSort
         'Bevor ein Node aus dem Hashset in die Ergebnisliste verschoben wird,
         ' werden rekursiv all seine Vorgänger verschoben
         Public rankedTables As List(Of DataTable)
         Public tableSrc As HashSet(Of DataTable)
         Public Sub New(ByVal dts As IEnumerable(Of DataTable))
            tableSrc = New HashSet(Of DataTable)(dts)
            rankedTables = New List(Of DataTable)(tableSrc.Count)
            While tableSrc.Count > 0
               RecurseRankedTables(tableSrc.First)
            End While
         End Sub
         Public Sub RecurseRankedTables(ByVal tb As DataTable)
            If Not tableSrc.Remove(tb) Then Return 'gegen Kreisbezüge sichern
            tb.ParentTables.ForEach(AddressOf RecurseRankedTables)
            rankedTables.Add(tb)
         End Sub
      End Class

   End Module

End Namespace