﻿Imports System.Linq
Imports System.Collections
Imports System.Collections.Generic
Imports System.Data
Imports System.Diagnostics
Imports System.Runtime.CompilerServices
Imports System.ComponentModel
Namespace System.Windows.Forms

   Public Module BindingSourceX

      ''' <summary>
      ''' provides a placeholder-syntax for filters. Sample: dv.FilterX("Datum >= ? And ? >= Datum", dtpVon.Value, dtpBis.Value)
      ''' </summary>
      ''' <exception cref="ArgumentException">expression contains more placeholders than values are passed</exception>
      <Extension()> _
      Public Sub FilterX(ByVal bs As BindingSource, ByVal expression As String, ByVal ParamArray values() As Object)
         bs.Filter = GetFilterString(expression, values)
      End Sub

      ''' <summary>
      ''' provides a placeholder-syntax for filters. Sample: dv.FilterX("Datum >= ? And ? >= Datum", dtpVon.Value, dtpBis.Value)
      ''' </summary>
      ''' <exception cref="ArgumentException">expression contains more placeholders than values are passed</exception>
      <Extension()> _
      Public Sub FilterX(ByVal dv As DataView, ByVal expression As String, ByVal ParamArray values() As Object)
         dv.RowFilter = GetFilterString(expression, values)
      End Sub

      Private Function GetFilterString(ByVal expression As String, ByVal values() As Object) As String
         Dim splits = expression.Split("?"c)      ' identify placeholder '?'
         If splits.Count > values.Length + 1 Then Throw New ArgumentException( _
            "expression contains more placeholders than values are passed")
         For i = 0 To splits.Length - 2
            Dim val = values(i)
            Dim format = If(TypeOf val Is String, "'{0}'", If(TypeOf val Is Date, "#{0}#", "{0}"))
            splits(i) &= String.Format(Globalization.CultureInfo.InvariantCulture, format, val)
         Next
         Return String.Concat(splits)
      End Function

      <Extension()> _
      Public Sub ListChangedEnabled(ByVal bs As BindingSource, ByVal value As Boolean)
         bs.RaiseListChangedEvents = value
         If value Then bs.ResetBindings(False)
      End Sub

      <Extension()> _
      Public Sub ChangeBinding(ByVal bs As BindingSource, ByVal dataSource As Object, ByVal datamember As String)
         With bs
            .RaiseListChangedEvents = False
            .DataSource = dataSource
            .DataMember = datamember
            .RaiseListChangedEvents = True
            .ResetBindings(False)
         End With
      End Sub

      <Extension()> _
      Public Function [Dataset](ByVal bs As BindingSource) As DataSet
         Dim BindSource As BindingSource = Nothing
         Do Until bs Is Nothing
            BindSource = bs
            bs = TryCast(bs.DataSource, BindingSource)
         Loop
         Dim src = BindSource.DataSource
         Dim ds = TryCast(src, DataSet)
         If ds.NotNull Then Return ds
         Dim tb = TryCast(src, DataTable)
         If tb.NotNull Then Return tb.DataSet
         Dim rw = TryCast(src, DataRow)
         If rw.NotNull Then Return rw.Table.DataSet
         Dim drv = TryCast(src, DataRowView)
         If drv.NotNull Then Return drv.Row.Table.DataSet
         If TypeOf src Is DataView Then Throw New Exception( _
            "DatasetFromBindingSource: Dont set BindingSources DataSource on DataView. It's buggy!")
         Return Nothing
      End Function

      <Extension()> _
      Public Function [DataTable](ByVal Source As BindingSource) As DataTable
         While Source.DataMember = ""
            Dim drv = TryCast(Source.DataSource, DataRowView)
            If drv.NotNull Then Return drv.Row.Table
            Source = TryCast(Source.DataSource, BindingSource)
            If Source.Null Then Return Nothing
         End While
         Dim DTS As DataSet = Dataset(Source)
         If DTS Is Nothing Then Return Nothing
         Dim SourceDataMember As String = Source.DataMember
         If DTS.Relations.Contains(SourceDataMember) Then
            Return DTS.Relations(SourceDataMember).ChildTable
         ElseIf DTS.Tables.Contains(SourceDataMember) Then
            Return DTS.Tables(SourceDataMember)
         Else
            Throw New Exception(String.Concat("Im Dataset anhand des BindingSource-Datamembers '", _
             SourceDataMember, "' keine Tabelle gefunden"))
         End If
      End Function

      <Extension()> _
      Public Function All(Of T As DataRow)(ByVal subj As BindingSource) As IEnumerable(Of T)
         Return From drv In subj.Cast(Of DataRowView)() Select DirectCast(drv.Row, T)
      End Function

      <Extension()> _
      Public Function All(Of T As DataRow)(ByVal subj As DataView) As IEnumerable(Of T)
         Return From drv In subj.Cast(Of DataRowView)() Select DirectCast(drv.Row, T)
      End Function

      ''' <summary>
      ''' returnt die Datarow am index. Bei ungültigem index Nothing (keine OutOfRange-Exception!)
      ''' </summary>
      <Extension()> _
      Public Function At(Of T As DataRow)(ByVal subj As DataView, ByVal index As Integer) As T
         If index < 0 OrElse index >= subj.Count Then Return Nothing
         Return DirectCast(DirectCast(subj(index), DataRowView).Row, T)
      End Function

      ''' <summary> returnt die typisierte Datarow an aktueller Position - oder Nothing. </summary>
      <Extension()> _
      Public Function At(Of T As DataRow)(ByVal subj As BindingSource) As T
         Return DirectCast(subj.At(subj.Position), T)
      End Function

      ''' <summary>
      ''' returnt die typisierte Datarow am index. Bei ungültigem index Nothing (keine OutOfRange-Exception!)
      ''' </summary>
      <Extension()> _
      Public Function At(Of T As DataRow)(ByVal subj As BindingSource, ByVal index As Integer) As T
         Return DirectCast(subj.At(index), T)
      End Function

      ''' <summary> returnt die untypisierte Datarow an aktueller Position. </summary>
      <Extension()> _
      Public Function At(ByVal subj As BindingSource) As DataRow
         Return subj.At(subj.Position)
      End Function

      ''' <summary>
      ''' returnt die Datarow am index. Bei ungültigem index Nothing (keine OutOfRange-Exception!)
      ''' </summary>
      <Extension()> _
      Public Function At(ByVal subj As BindingSource, ByVal index As Integer) As DataRow
         If index < 0 OrElse index >= subj.Count Then Return Nothing
         Return DirectCast(subj(index), DataRowView).Row
      End Function

      <Extension()> _
      Public Function FindX(Of T As DataRow)(ByVal subj As BindingSource, ByVal predicate As Func(Of T, Boolean)) As Integer
         For i As Integer = 0 To subj.Count - 1
            If predicate(DirectCast(DirectCast(subj(i), DataRowView).Row, T)) Then Return i
         Next
         Return -1
      End Function

      <Extension()> _
      Public Function FindX(ByVal subj As BindingSource, ByVal columnName As String, ByVal Key As Object) As Integer
         'BindingSource.Find funzt nicht bei relateten BindingSourcses
         'FindX = subj.Find(PropertyName, Key)
         For i As Integer = 0 To subj.Count - 1
            If DirectCast(subj(i), DataRowView)(columnName).Equals(Key) Then Return i
         Next
         Return -1
      End Function

      Private Function KeysMatch(ByVal row As DataRow, ByVal cols As DataColumn(), ByVal keys As Object()) As Boolean
         For ii = 0 To cols.Length - 1
            If Not Object.Equals(keys(ii), row(cols(ii))) Then Return False
         Next
         Return True
      End Function

      ''' <summary> stellt den Datensatz mit den angegebenen Primärschlüssel-Werten ein </summary>
      <Extension()> _
      Public Function MoveTo(ByVal subj As BindingSource, ByVal primKeys() As Object) As Boolean
         If subj.Count = 0 Then Return False
         Dim rw = DirectCast(subj(0), DataRowView).Row
         Dim cols = rw.Table.PrimaryKey
         Dim i = 0
         Try
            If KeysMatch(rw, cols, primKeys) Then Return True
            For i = 1 To subj.Count - 1
               rw = DirectCast(subj(i), DataRowView).Row
               If KeysMatch(rw, cols, primKeys) Then Return True
            Next
         Finally
            subj.Position = i
         End Try
         Return False
      End Function

      ''' <summary> stellt den Datensatz mit dem angegebenen Primärschlüssel-Wert ein </summary>
      <Extension()> _
      Public Function MoveTo(ByVal subj As BindingSource, ByVal primKey As Object) As Boolean
         If subj.Count = 0 Then Return False
         Return subj.MoveTo(DirectCast(subj(0), DataRowView).Row.Table.PrimaryKey(0).ColumnName, primKey)
      End Function

      ''' <summary> 
      ''' stellt den ersten Datensatz ein, dessen Wert in der angegebenen spalte mit Key übereinstimmt
      ''' </summary>
      <Extension()> _
      Public Function MoveTo(ByVal subj As BindingSource, ByVal columnName As String, ByVal Key As Object) As Boolean
         Dim i = subj.FindX(columnName, Key)
         MoveTo = i >= 0
         subj.Position = i
      End Function

      ''' <summary> stellt die angegebene DataRow ein </summary>
      <Extension()> _
      Public Function MoveTo(ByVal subj As BindingSource, ByVal row As DataRow) As Boolean
         Dim dv = DirectCast(subj.List, DataView)
         For i As Integer = 0 To subj.Count - 1
            If dv(i).Row Is row Then
               subj.CancelEdit() 'ansonsten setzter u.U. beim Moven die vorherige Row auf Rowstate.Changed
               subj.Position = i
               Return True
            End If
         Next
         Return False
      End Function

      ''' <summary> stellt die erste matchende DataRow ein </summary>
      <Extension()> _
      Public Function MoveTo(Of T As DataRow)(ByVal subj As BindingSource, ByVal predicate As Func(Of T, Boolean)) As Boolean
         Dim dv = DirectCast(subj.List, DataView)
         For i As Integer = 0 To subj.Count - 1
            If predicate(DirectCast(dv(i).Row, T)) Then
               subj.Position = i
               Return True
            End If
         Next
         Return False
      End Function

      <Diagnostics.DebuggerStepThrough()> _
      <Extension()> _
      Public Function AddNewX(Of T As DataRow)(ByVal subj As BindingSource) As T
         Return DirectCast(DirectCast(subj.AddNew(), DataRowView).Row, T)
      End Function

      ''' <summary> returnt eine typisierte Enumeration aller Datarows </summary>
      <Extension()> _
      Public Function GetRows(Of T As DataRow)(ByVal subj As BindingSource) As IEnumerable(Of T)
         Return subj.Cast(Of DataRowView).Select(Function(drv) DirectCast(drv.Row, T))
      End Function

      <Diagnostics.DebuggerStepThrough()> _
      <Extension()> _
      Public Function RowX(Of T As DataRow)(ByVal subj As DataRowView) As T
         Return DirectCast(subj.Row, T)
      End Function

      <Diagnostics.DebuggerStepThrough()> _
      <Extension()> _
      Public Function AddNewX(Of T)(ByVal subj As IBindingList) As T
         Return DirectCast(subj.AddNew(), T)
      End Function

      ''' <summary> for debugging-purposes </summary>
      Public Function CheckChanges(ByVal dts As DataSet) As Boolean
         dts = dts.GetChanges
         If dts.Null Then MsgBox("no Changes") : Return False
         Dim tbls = dts.Tables.Cast(Of DataTable)().ToArray
         Dim rws = tbls.SelectMany(Function(tb) tb.Rows.Cast(Of DataRow)())
         Dim infos = From tbl In tbls From rw In tbl.Rows.Cast(Of DataRow)() _
            Select String.Concat(rw.GetType.Name, _
            String.Concat(rw.ItemArray.Select(Function(itm) " " & itm.ToString).ToArray))
         MsgBox(String.Join(vbNewLine, infos.ToArray))
         Return True
      End Function

   End Module

End Namespace
