﻿Imports System.ComponentModel
Imports System.Windows.Forms

Namespace System.Data

   ''' <summary>some common basics of DatasetXmlAdapter and DatasetAdapter</summary>
   Public MustInherit Class DatasetAdapterBase

      Public Shared ReadOnly Attachs As New ComponentProperty(Of DataSet, DatasetAdapterBase)

      Protected _DataSet As DataSet
      Public HasChanges As Func(Of Boolean)
      Public ReadOnly HandleFormClosing As FormClosingEventHandler = AddressOf _HandleFormClosing

      Protected MustOverride Sub _Save(ByVal tbls As IEnumerable(Of DataTable))
      Protected MustOverride Sub _Fill(ByVal tbls As IEnumerable(Of DataTable))
      Public Sub Fill(ByVal tbls As Collections.Generic.IEnumerable(Of DataTable))
         If tbls.FirstOrDefault.Null Then
            DataSet.Clear()
         Else
            tbls.ForEach(AddressOf ClearRecursive)
         End If
         _Fill(tbls)
         DataSet.AcceptChanges() 'maybe brain again about particular AcceptChanges for DataSetAdapter
      End Sub
      Public Overridable Property DataSet() As DataSet
         Get
            Return _DataSet
         End Get
         Set(ByVal value As DataSet)
            If _DataSet Is value Then Return
            _DataSet = value
            HasChanges = AddressOf _HasChanges
         End Set
      End Property
      Public Sub Save(ByVal form As Form, Optional ByVal checkForChanges As Boolean = True)
         If form.NotNull AndAlso Not form.Validate() Then
            System.Media.SystemSounds.Hand.Play()
            Return
         End If
         Dim atts = TableAttachs(Persistance.YesOrGuess).ToList
         For Each bs In atts.SelectMany(Function(att) att.BindingSources)
            bs.EndEdit()
         Next
         System.Media.SystemSounds.Asterisk.Play()
         If checkForChanges AndAlso Not HasChanges.Invoke() Then Return
         _Save(atts.Select(Function(att) att.Table))
         DataSet.AcceptChanges()
      End Sub
      Protected Function TableAttachs(ByVal persistance As Persistance) _
            As IEnumerable(Of DataTableAttach)
         Return From tb In _DataSet.GetRankedTables Let att = tb.Attach _
            Where 0 <> (att.Persistance And persistance) Select att
      End Function
      Private Function _HasChanges() As Boolean
         For Each att In TableAttachs(Persistance.No)
            att.Table.AcceptChanges()
         Next
         Return _DataSet.HasChanges
      End Function

      Private Sub _HandleFormClosing(ByVal sender As Object, ByVal e As CancelEventArgs)
         If e.Cancel Then Return
         Dim frm = DirectCast(sender, Form)
         If Not HasChanges.Invoke() Then Return
         Select Case MessageBox.Show(frm, "Änderungen speichern?", "Das Ende ist nahe", MessageBoxButtons.YesNoCancel)
            Case Windows.Forms.DialogResult.Yes
               Save(frm, False)
            Case Windows.Forms.DialogResult.Cancel
               e.Cancel = True
               Return
         End Select
         e.Cancel = False
      End Sub

   End Class

End Namespace
